@@ -36,7 +36,7 @@
 
 
       RETURN
-   END SUBROUTINE 
+   END SUBROUTINE
 
 
 
@@ -105,45 +105,45 @@
       COMPLEX(DP),  INTENT(IN)  :: sfac(:,:)
       REAL(DP),     INTENT(IN)  :: epseu
 
-      INTEGER     :: ig,k,is, ispin, s_ngm_ 
+      INTEGER     :: ig,k,is, ispin, s_ngm_
       COMPLEX(DP) :: dsvp, svp, depst(6), depst1,depst2,depst3,depst4,depst5,depst6
       REAL(DP)    :: wz
       !
-      CALL start_clock("stress_local") 
+      CALL start_clock("stress_local")
       s_ngm_ = dffts%ngm
-DEV_ACC data present(rhoe, drhoe,sfac) copyin(gagb,vps,dvps) 
-DEV_ACC update self(drhoe(1,1:6), sfac(1,1:nsp)) 
+!$acc data present(rhoe, drhoe,sfac) copyin(gagb,vps,dvps)
+!$acc update self(drhoe(1,1:6), sfac(1,1:nsp))
       depst  = (0.d0,0.d0)
-      depst1 = (0.d0,0.d0) 
+      depst1 = (0.d0,0.d0)
       depst2 = (0.d0,0.d0)
-      depst3 = (0.d0,0.d0) 
-      depst4 = (0.d0,0.d0) 
-      depst5 = (0.d0,0.d0) 
-      depst6 = (0.d0,0.d0) 
+      depst3 = (0.d0,0.d0)
+      depst4 = (0.d0,0.d0)
+      depst5 = (0.d0,0.d0)
+      depst6 = (0.d0,0.d0)
       wz = 2.0d0
-DEV_ACC parallel loop private(svp) reduction(+:depst1,depst2,depst3,depst4,depst5,depst6) 
+!$acc parallel loop private(svp) reduction(+:depst1,depst2,depst3,depst4,depst5,depst6)
       DO ig = gstart, s_ngm_
          svp = 0.0d0
-DEV_ACC loop seq 
+!$acc loop seq
          DO is = 1, nsp
             svp = svp + sfac( ig, is ) * vps( ig, is )
          END DO
          depst1 = depst1 + wz * CONJG( drhoe( ig, 1 ) ) * svp
-         depst2 = depst2 + wz * CONJG( drhoe( ig, 2 ) ) * svp         
+         depst2 = depst2 + wz * CONJG( drhoe( ig, 2 ) ) * svp
          depst3 = depst3 + wz * CONJG( drhoe( ig, 3 ) ) * svp
          depst4 = depst4 + wz * CONJG( drhoe( ig, 4 ) ) * svp
          depst5 = depst5 + wz * CONJG( drhoe( ig, 5 ) ) * svp
          depst6 = depst6 + wz * CONJG( drhoe( ig, 6 ) ) * svp
       END DO
       !
-      depst = [depst1,depst2,depst3,depst4,depst5,depst6] 
-      depst1 = (0.d0,0.d0) 
-      depst2 = (0.d0,0.d0)  
-      depst3 = (0.d0,0.d0) 
-      depst4 = (0.d0,0.d0) 
-      depst5 = (0.d0,0.d0) 
-      depst6 = (0.d0,0.d0) 
-      
+      depst = [depst1,depst2,depst3,depst4,depst5,depst6]
+      depst1 = (0.d0,0.d0)
+      depst2 = (0.d0,0.d0)
+      depst3 = (0.d0,0.d0)
+      depst4 = (0.d0,0.d0)
+      depst5 = (0.d0,0.d0)
+      depst6 = (0.d0,0.d0)
+
 
       IF( gstart == 2 ) THEN
          svp = 0.0d0
@@ -151,12 +151,12 @@ DEV_ACC loop seq
             svp = svp + sfac( 1, is ) * vps( 1, is )
          END DO
          depst = depst + CONJG( drhoe( 1, : ) ) * svp
-      END IF 
+      END IF
       !
-DEV_ACC parallel loop private(dsvp) reduction(+:depst1,depst2,depst3,depst4,depst5,depst6) 
+!$acc parallel loop private(dsvp) reduction(+:depst1,depst2,depst3,depst4,depst5,depst6)
       DO ig = gstart, s_ngm_
          dsvp = 0.0d0
-DEV_ACC loop seq 
+!$acc loop seq
          DO is = 1, nsp
             dsvp = dsvp + sfac( ig, is ) * dvps( ig, is )
          END DO
@@ -167,10 +167,10 @@ DEV_ACC loop seq
          depst5 = depst5 - wz * 2.0d0 * CONJG( rhoe( ig ) ) * dsvp * gagb( 5, ig )
          depst6 = depst6 - wz * 2.0d0 * CONJG( rhoe( ig ) ) * dsvp * gagb( 6, ig )
       END DO
-      depst = depst + [depst1,depst2,depst3,depst4,depst5,depst6] 
+      depst = depst + [depst1,depst2,depst3,depst4,depst5,depst6]
       deps = omega * DBLE( depst )
-DEV_ACC end data  
-      CALL stop_clock("stress_local") 
+!$acc end data
+      CALL stop_clock("stress_local")
       RETURN
    END SUBROUTINE stress_local_x
 
@@ -178,16 +178,16 @@ DEV_ACC end data
 
 
 !------------------------------------------------------------------------------!
-   SUBROUTINE stress_kin_x( dekin, c0_bgrp, occ_bgrp ) 
+   SUBROUTINE stress_kin_x( dekin, c0_bgrp, occ_bgrp )
 !------------------------------------------------------------------------------!
-      !! This routine computes the kinetic energy contribution to the stress 
+      !! This routine computes the kinetic energy contribution to the stress
       !! tensor.
       !
-      !! \[ \text{dekin}(:) = - 2 (\sum_i) f(i) \cdot 
+      !! \[ \text{dekin}(:) = - 2 (\sum_i) f(i) \cdot
       !!    (\sum_g) \text{gagb}(:,g) \text{c0}(g,i)^*\cdot \text{c0}(g,i) \]
 
       !
-      !  dekin(:) = - 2 (sum over i) f(i) * 
+      !  dekin(:) = - 2 (sum over i) f(i) *
       !    ( (sum over g) gagb(:,g) CONJG( c0(g,i) ) c0(g,i) )
       !
 
@@ -215,7 +215,7 @@ DEV_ACC end data
 !  ----------------------------------------------
 
       dekin = 0.0_DP
-      ALLOCATE( arg( ngw ) ) 
+      ALLOCATE( arg( ngw ) )
 
       efac = 2.0d0 * qcutz / q2sigma / SQRT(pi)
       IF( efac > 0.0d0 ) THEN
@@ -245,7 +245,7 @@ DEV_ACC end data
         END DO
       END DO
       dekin = - 2.0_DP * dekin
-      DEALLOCATE(arg) 
+      DEALLOCATE(arg)
       RETURN
    END SUBROUTINE stress_kin_x
 
@@ -271,31 +271,31 @@ DEV_ACC end data
       INTEGER     :: ij, is, ig
       COMPLEX(DP) :: drhop
       !
-DEV_ACC data present(drhot, sfac) copyin(gagb) 
+!$acc data present(drhot, sfac) copyin(gagb)
       DO ij = 1, 6
          IF( dalbe( ij ) > 0.0d0 ) THEN
-DEV_ACC kernels async 
+!$acc kernels async
             DO is = 1, nsp
                DO ig = 1, dffts%ngm
                   drhot(ig,ij) = drhot(ig,ij) - sfac(ig,is)*rhops(ig,is)
                ENDDO
             END DO
-DEV_ACC end kernels 
+!$acc end kernels
          END IF
       END DO
-DEV_ACC parallel loop private(drhop) 
+!$acc parallel loop private(drhop)
       DO ig = 1, dffts%ngm
          drhop = 0.0d0
-DEV_ACC loop seq 
+!$acc loop seq
          DO is = 1, nsp
            drhop = drhop - sfac( ig, is ) * rhops(ig,is) * rcmax(is)**2 * 0.5D0
          END DO
-DEV_ACC loop seq 
+!$acc loop seq
          DO ij = 1, 6
              drhot(ig,ij) = drhot(ig,ij) - drhop * gagb( ij, ig )
          END DO
       END DO
-DEV_ACC end data
+!$acc end data
       RETURN
    END SUBROUTINE add_drhoph_x
 
@@ -303,7 +303,7 @@ DEV_ACC end data
 
 
 !------------------------------------------------------------------------------!
-   SUBROUTINE stress_har_x(deht, ehr, sfac, rhoeg, gagb, omega ) 
+   SUBROUTINE stress_har_x(deht, ehr, sfac, rhoeg, gagb, omega )
 !------------------------------------------------------------------------------!
 
       use kinds,              only: DP
@@ -363,7 +363,7 @@ DEV_ACC end data
       !
       CALL add_drhoph( drhot, sfac, gagb )
 
-      CALL stress_hartree(deht, ehr, sfac, rhot, drhot, gagb, omega ) 
+      CALL stress_hartree(deht, ehr, sfac, rhot, drhot, gagb, omega )
 
       DEALLOCATE( rhot, drhot )
 
@@ -374,18 +374,18 @@ DEV_ACC end data
 
 
 !------------------------------------------------------------------------------!
-   SUBROUTINE stress_hartree_x(deht, ehr, sfac, rhot, drhot, gagb, omega ) 
+   SUBROUTINE stress_hartree_x(deht, ehr, sfac, rhot, drhot, gagb, omega )
 !------------------------------------------------------------------------------!
       !! This subroutine computes:
       !
       !! \[ d E_\text{hartree} / dh  =
-      !!    E_\text{hartree} \cdot h^t + 
+      !!    E_\text{hartree} \cdot h^t +
       !!    4\pi \Omega \rho_t \cdot \rho_t^* / G^2 / G^2 \cdot G_\alpha \cdot G_\beta +
       !!    4\pi \Omega \text{Re}[ \rho_t^* \cdot d\rho_t / G^2 ] \]
       !
-      !! where:  
-      !! \( \rho_t  = \rho_e + \rho_I \);  
-      !! \( d\rho_t = d\rho_t \);  
+      !! where:
+      !! \( \rho_t  = \rho_e + \rho_I \);
+      !! \( d\rho_t = d\rho_t \);
       !! \( dh = -\rho_e + d\rho_\text{hard} / dh + d\rho_I / dh \).
       !
       use kinds,              only: DP
@@ -407,7 +407,7 @@ DEV_ACC end data
       COMPLEX(DP) :: drhot(:,:)
       COMPLEX(DP), INTENT(IN) :: sfac(:,:)
 
-      COMPLEX(DP)    DEHC(6), dehc1, dehc2, dehc3, dehc4, dehc5, dehc6 
+      COMPLEX(DP)    DEHC(6), dehc1, dehc2, dehc3, dehc4, dehc5, dehc6
       COMPLEX(DP)    CFACT
       REAL(DP), ALLOCATABLE :: hgm1( : )
       REAL(DP)    :: wz
@@ -415,34 +415,34 @@ DEV_ACC end data
       INTEGER       ig, is, k, iss, p_ngm_
 
       DEHC  = (0.D0,0.D0)
-      dehc1 = cmplx(0.d0,0.d0,kind=dp) 
-      dehc2 = cmplx(0.d0,0.d0,kind=dp) 
-      dehc3 = cmplx(0.d0,0.d0,kind=dp) 
-      dehc4 = cmplx(0.d0,0.d0,kind=dp) 
-      dehc5 = cmplx(0.d0,0.d0,kind=dp) 
-      dehc6 = cmplx(0.d0,0.d0,kind=dp) 
-    
+      dehc1 = cmplx(0.d0,0.d0,kind=dp)
+      dehc2 = cmplx(0.d0,0.d0,kind=dp)
+      dehc3 = cmplx(0.d0,0.d0,kind=dp)
+      dehc4 = cmplx(0.d0,0.d0,kind=dp)
+      dehc5 = cmplx(0.d0,0.d0,kind=dp)
+      dehc6 = cmplx(0.d0,0.d0,kind=dp)
+
       DEHT  = 0.D0
       p_ngm_ = dfftp%ngm
 
       wz = 2.0d0
 
       ALLOCATE( hgm1( p_ngm_ ) )
-      ! 
-DEV_ACC data present(sfac,rhot,drhot) copyin(gagb,gg) create(hgm1)
       !
-DEV_ACC kernels  
+!$acc data present(sfac,rhot,drhot) copyin(gagb,gg) create(hgm1)
+      !
+!$acc kernels
       hgm1( 1 ) = 0.0d0
-DEV_ACC loop 
+!$acc loop
       DO ig = gstart, p_ngm_
          hgm1( ig ) = 1.D0 / gg(ig) / tpiba2
       END DO
-DEV_ACC end kernels
+!$acc end kernels
 
       ! Add term  rho_t * CONJG( rho_t ) / G^2 * G_alpha * G_beta / G^2
-DEV_ACC parallel loop private(cfact) reduction(+:dehc1,dehc2,dehc3,dehc4,dehc5,dehc6) 
+!$acc parallel loop private(cfact) reduction(+:dehc1,dehc2,dehc3,dehc4,dehc5,dehc6)
       DO ig = gstart, p_ngm_
-         cfact = rhot( ig ) * CONJG( rhot( ig ) ) * hgm1( ig ) ** 2 
+         cfact = rhot( ig ) * CONJG( rhot( ig ) ) * hgm1( ig ) ** 2
          dehc1 = dehc1 + cfact * gagb(1,ig)
          dehc2 = dehc2 + cfact * gagb(2,ig)
          dehc3 = dehc3 + cfact * gagb(3,ig)
@@ -452,7 +452,7 @@ DEV_ACC parallel loop private(cfact) reduction(+:dehc1,dehc2,dehc3,dehc4,dehc5,d
       END DO
 
       ! Add term  2 * Re{ CONJG( rho_t ) * drho_t / G^2 }
-DEV_ACC parallel loop reduction(+:dehc1,dehc2,dehc3,dehc4,dehc5,dehc6) 
+!$acc parallel loop reduction(+:dehc1,dehc2,dehc3,dehc4,dehc5,dehc6)
       DO ig = gstart, p_ngm_
          dehc1 = dehc1  +  rhot( ig ) * CONJG( drhot( ig, 1 ) ) * hgm1( ig )
          dehc2 = dehc2  +  rhot( ig ) * CONJG( drhot( ig, 2 ) ) * hgm1( ig )
@@ -469,7 +469,7 @@ DEV_ACC parallel loop reduction(+:dehc1,dehc2,dehc3,dehc4,dehc5,dehc6)
       else
         deht = wz * fpi * omega * DBLE(dehc)
       end if
-DEV_ACC end data
+!$acc end data
       DEALLOCATE( hgm1 )
       RETURN
    END SUBROUTINE stress_hartree_x
@@ -595,9 +595,9 @@ DEV_ACC end data
          REAL(DP), INTENT(IN)  :: tpiba2
 
          INTEGER :: k, ig
-      
+
 !$omp parallel do default(shared), private(k)
-         DO ig = 1, ngm          
+         DO ig = 1, ngm
             DO k = 1, 6
                gagb( k, ig ) = g( alpha( k ), ig ) * g( beta( k ), ig ) * tpiba2
             END DO
