@@ -5,7 +5,6 @@ cd example01/
 ./run_example
 ./compare.sh
 echo "DONE"
-rm -fr home/nicola/Scrivania/CODES/git/koopmans/quantum_espresso/qe_koopmans/tempdir
 cd $home
 
 echo "Running example02"
@@ -13,7 +12,6 @@ cd example02/
 ./run_example
 ./compare.sh
 echo "DONE"
-rm -fr home/nicola/Scrivania/CODES/git/koopmans/quantum_espresso/qe_koopmans/tempdir
 cd $home 
 
 echo "Running example03"
@@ -21,5 +19,26 @@ cd example03/
 ./run_example
 ./compare.sh
 echo "DONE"
-rm -fr home/nicola/Scrivania/CODES/git/koopmans/quantum_espresso/qe_koopmans/tempdir
 cd $home 
+
+echo "Running example04"
+cd example04/
+./run_example
+./compare.sh
+echo "DONE"
+cd $home 
+
+echo "Running example05"
+cd example05/
+./run_example
+./compare.sh
+echo "DONE"
+cd $home
+
+echo "Running example06"
+cd example06/
+./run_example
+./compare.sh
+echo "DONE"
+cd $home
+
