@@ -30,6 +30,9 @@ MODULE dft_setting_routines
             xclib_finalize_libxc
   PUBLIC :: set_libxc_ext_param, get_libxc_ext_param
   PUBLIC :: capital
+#if (__LIBXC && XC_MAJOR_VERSION < 5)
+  INTEGER,PARAMETER :: XC_FAMILY_HYB_LDA=-1, XC_FAMILY_HYB_GGA=-1, XC_FAMILY_HYB_MGGA=-1
+#endif 
   !
 CONTAINS
   !
